import boto3
import time
import sys
import os
from datetime import datetime

def s3_folder_exists(bucket, folder, event_date, event_hour):
    print(f"Check data availability for {bucket}, {folder}, event_date = {event_date}, event_hour = {event_hour}")
    session = boto3.session.Session()
    client = session.client('s3')

    prefix = folder+'/event_date='+event_date+'/event_hour='+event_hour
    response = client.list_objects_v2(
        Bucket=bucket,
        Prefix=prefix,
    )
    return len(response['Contents']) > 0

def add_partition(aws_region, database, table_name, s3_bucket, folder, athena_result_bucket, event_date, event_hour):
    print(f"add_partition {table_name}, {aws_region} begin")
    session = boto3.session.Session(region_name=aws_region)
    client = session.client('athena')
    sql = "ALTER TABLE " + table_name + " ADD PARTITION (event_date= '" \
          + event_date + "', event_hour= '" + event_hour + "') LOCATION " \
          + "'s3://" + s3_bucket + "/" + folder + "/event_date=" \
          + event_date + "/event_hour=" + event_hour + "/'"

    response = client.start_query_execution(
        QueryString=sql,
        QueryExecutionContext={
            'Database': database
        },
        ResultConfiguration={
            'OutputLocation': 's3://' + athena_result_bucket
        },
        WorkGroup=os.environ['EEDR_ATHENA_WORKGROUP']
    )
    print(sql)
    return response

def main():
    print(f"Starting at {datetime.now()}")
    database_name = os.environ['DATABASE_NAME']
    aws_region = os.environ['AWS_REGION']
    athena_result_bucket = f"{os.environ['EEDR_ATHENA_OUTPUT']}/results"
    s3_bucket = os.environ['EEDR_OUTPUT_BUCKET']
    es_s3_bucket = os.environ['ES_OUTPUT_BUCKET']
    now = time.gmtime()
    event_date = time.strftime("%Y-%m-%d", now)
    event_hour = time.strftime("%H", now)

    table_name = 'flink_eedr_alert'
    folder = 'alert/event-simple'
    try:
        if s3_folder_exists(s3_bucket, folder, event_date, event_hour):
            add_partition(aws_region, database_name, table_name, s3_bucket, folder, athena_result_bucket, event_date, event_hour)
    except Exception as error:
        print("An exception occurred:", error)

    table_name = 'flink_eedr_event'
    folder = 'eedr/event-simple'
    try:
        if s3_folder_exists(s3_bucket, folder, event_date, event_hour):
            add_partition(aws_region, database_name, table_name, s3_bucket, folder, athena_result_bucket, event_date, event_hour)
    except Exception as error:
        print("An exception occurred:", error)

    table_name = 'flink_eedr_sample'
    folder = 'sample/event-simple'
    try:
        if s3_folder_exists(s3_bucket, folder, event_date, event_hour):
            add_partition(aws_region, database_name, table_name, s3_bucket, folder, athena_result_bucket, event_date, event_hour)
    except Exception as error:
        print("An exception occurred:", error)

    table_name = 'flink_eedr_random_sample'
    folder = 'eedr-random-sample/event-simple'
    try:
        if s3_folder_exists(s3_bucket, folder, event_date, event_hour):
            add_partition(aws_region, database_name, table_name, s3_bucket, folder, athena_result_bucket, event_date, event_hour)
    except Exception as error:
        print("An exception occurred:", error)

    table_name = 'flink_eedr_popularity_filter'
    folder = 'popularity-filter'
    try:
        if s3_folder_exists(s3_bucket, folder, event_date, event_hour):
            add_partition(aws_region, database_name, table_name, s3_bucket, folder, athena_result_bucket, event_date, event_hour)
    except Exception as error:
        print("An exception occurred:", error)

    table_name = 'flink_es_event'
    folder = 'es-events/event-simple'
    try:
        if s3_folder_exists(es_s3_bucket, folder, event_date, event_hour):
            add_partition(aws_region, database_name, table_name, es_s3_bucket, folder, athena_result_bucket, event_date, event_hour)
    except Exception as error:
        print("An exception occurred:", error)

    print(f"End at {datetime.now()}")

main()
